package com.chelvc.framework.base.group;

import java.util.Collection;
import java.util.Collections;
import java.util.Map;
import java.util.stream.Collectors;

import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.model.Caps;
import com.chelvc.framework.common.util.ObjectUtils;
import com.google.common.collect.Maps;
import lombok.NonNull;

/**
 * 基于内存场景分组存储实现
 *
 * @author Woody
 * @date 2024/11/24
 */
public class MemoryGroupStore implements GroupStore {
    private final Map<Long, Map<String, Caps>> cache = Maps.newConcurrentMap();

    @Override
    public Caps get(@NonNull String scene) {
        Long id = SessionContextHolder.getId();
        if (id == null) {
            return null;
        }
        return ObjectUtils.ifNull(this.cache.get(id), groups -> groups.get(scene));
    }

    @Override
    public Map<String, Caps> get(@NonNull Collection<String> scenes) {
        if (ObjectUtils.isEmpty(scenes)) {
            return Collections.emptyMap();
        }
        Long id = SessionContextHolder.getId();
        if (id == null) {
            return Collections.emptyMap();
        }
        Map<String, Caps> groups = ObjectUtils.ifNull(this.cache.get(id), Collections::emptyMap);
        return scenes.stream().collect(Collectors.toMap(scene -> scene, groups::get));
    }

    @Override
    public Caps set(@NonNull String scene, @NonNull Caps group) {
        Long id = SessionContextHolder.getId();
        if (id == null) {
            return group;
        }
        Map<String, Caps> groups = this.cache.get(id);
        if (groups == null) {
            groups = this.cache.computeIfAbsent(id, k -> Maps.newConcurrentMap());
        }
        return groups.computeIfAbsent(scene, k -> group);
    }

    @Override
    public Map<String, Caps> set(@NonNull Map<String, Caps> groups) {
        if (ObjectUtils.isEmpty(groups)) {
            return Collections.emptyMap();
        }
        Map<String, Caps> values = Maps.newHashMapWithExpectedSize(groups.size());
        groups.forEach((scene, group) -> values.put(scene, this.set(scene, group)));
        return values;
    }
}
