package com.chelvc.framework.security.context;

import javax.crypto.Cipher;

import com.chelvc.framework.base.context.Session;
import com.chelvc.framework.common.util.AESUtils;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.CodecUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.security.config.SecurityProperties;
import lombok.NonNull;

/**
 * 信息安全数据加解密上下文默认实现
 *
 * @author Woody
 * @date 2024/6/25
 */
public class DefaultSecurityCryptoContext implements SecurityCryptoContext {
    private final String key;
    private final String name;
    private final String secret;
    private final String iv;

    public DefaultSecurityCryptoContext(@NonNull SecurityProperties properties) {
        this.name = AESUtils.CBC_PKCS5PADDING;
        this.secret = properties.getSecret();
        this.iv = StringUtils.ifEmpty(this.secret, AESUtils::secret2iv);
        this.key = this.secret == null ? null : (this.name + this.secret + this.iv);
    }

    @Override
    public Cipher getCipher(int mode) {
        String secret = AssertUtils.nonempty(this.secret, () -> "AES secret is missing");
        String iv = AssertUtils.nonempty(this.iv, () -> "AES iv is missing");
        return AESUtils.lookupCipher(this.key, mode, () -> AESUtils.getCipher(this.name, mode, secret, iv));
    }

    @Override
    public String sign(@NonNull Session session, String payload) {
        String secret = AssertUtils.nonempty(this.secret, () -> "AES secret is missing");
        String plaintext = payload + secret + session.getDevice() + session.getPlatform() +
                session.getTerminal() + session.getVersion() + session.getTimestamp();
        return CodecUtils.md5(plaintext);
    }
}
